@@ -23,7 +23,6 @@ from __future__ import division, print_function, absolute_import
 import os
 import sys
 import time
-import functools
 import traceback
 from random import Random
 
@@ -41,12 +40,11 @@ from hypothesis.control import BuildContext
 from hypothesis._settings import settings as Settings
 from hypothesis._settings import Phase, Verbosity, HealthCheck, \
     note_deprecation
-from hypothesis.executors import new_style_executor, \
-    default_new_style_executor
+from hypothesis.executors import new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
 from hypothesis.internal.compat import ceil, str_to_bytes, \
-    get_type_hints, getfullargspec, encoded_filepath
+    benchmark_time, get_type_hints, getfullargspec, encoded_filepath
 from hypothesis.internal.coverage import IN_COVERAGE_TESTS
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.escalation import is_hypothesis_file, \
@@ -55,11 +53,12 @@ from hypothesis.internal.reflection import is_mock, proxies, nicerepr, \
     arg_string, impersonate, function_digest, fully_qualified_name, \
     define_function_signature, convert_positional_arguments, \
     get_pretty_function_description
+from hypothesis.internal.healthcheck import fail_health_check
 from hypothesis.internal.conjecture.data import Status, StopTest, \
     ConjectureData
 from hypothesis.searchstrategy.strategies import SearchStrategy
 from hypothesis.internal.conjecture.engine import ExitReason, \
-    ConjectureRunner, uniform, sort_key
+    ConjectureRunner, sort_key
 
 try:
     from coverage.tracer import CFileDisposition as FileDisposition
@@ -76,18 +75,6 @@ def new_random():
     return random.Random(random.getrandbits(128))
 
 
-def test_is_flaky(test, expected_repr):
-    @functools.wraps(test)
-    def test_or_flaky(*args, **kwargs):
-        text_repr = arg_string(test, args, kwargs)
-        raise Flaky(
-            (
-                'Hypothesis %s(%s) produces unreliable results: Falsified'
-                ' on the first call but did not on a subsequent one'
-            ) % (test.__name__, text_repr,))
-    return test_or_flaky
-
-
 @attr.s()
 class Example(object):
     args = attr.ib()
@@ -113,37 +100,6 @@ def example(*args, **kwargs):
     return accept
 
 
-def reify_and_execute(
-    search_strategy, test,
-    print_example=False,
-    is_final=False, collector=None
-):
-    def run(data):
-        with BuildContext(data, is_final=is_final):
-            import random as rnd_module
-            rnd_module.seed(0)
-            args, kwargs = data.draw(search_strategy)
-
-            if print_example:
-                report(
-                    lambda: 'Falsifying example: %s(%s)' % (
-                        test.__name__, arg_string(test, args, kwargs)))
-            elif current_verbosity() >= Verbosity.verbose:
-                report(
-                    lambda: 'Trying example: %s(%s)' % (
-                        test.__name__, arg_string(test, args, kwargs)))
-            if collector is None:
-                return test(*args, **kwargs)
-            else:  # pragma: no cover
-                try:
-                    collector.start()
-                    return test(*args, **kwargs)
-                finally:
-                    collector.stop()
-
-    return run
-
-
 def seed(seed):
     """seed: Start the test execution from a specific seed.
 
@@ -279,151 +235,6 @@ def execute_explicit_examples(
             raise
 
 
-def fail_health_check(settings, message, label):
-    # Tell pytest to omit the body of this function from tracebacks
-    # http://doc.pytest.org/en/latest/example/simple.html#writing-well-integrated-assertion-helpers
-    __tracebackhide__ = True
-
-    if label in settings.suppress_health_check:
-        return
-    if not settings.perform_health_check:
-        return
-    message += (
-        '\nSee https://hypothesis.readthedocs.io/en/latest/health'
-        'checks.html for more information about this. '
-    )
-    message += (
-        'If you want to disable just this health check, add %s '
-        'to the suppress_health_check settings for this test.'
-    ) % (label,)
-    raise FailedHealthCheck(message)
-
-
-def perform_health_checks(random, settings, test_runner, search_strategy):
-    # Tell pytest to omit the body of this function from tracebacks
-    __tracebackhide__ = True
-    if not settings.perform_health_check:
-        return
-    if not Settings.default.perform_health_check:
-        return
-
-    health_check_random = Random(random.getrandbits(128))
-    # We "pre warm" the health check with one draw to give it some
-    # time to calculate any cached data. This prevents the case
-    # where the first draw of the health check takes ages because
-    # of loading unicode data the first time.
-    data = ConjectureData(
-        max_length=settings.buffer_size,
-        draw_bytes=lambda data, n: uniform(health_check_random, n)
-    )
-    with Settings(settings, verbosity=Verbosity.quiet):
-        try:
-            test_runner(data, reify_and_execute(
-                search_strategy,
-                lambda *args, **kwargs: None,
-            ))
-        except BaseException:
-            pass
-    count = 0
-    overruns = 0
-    filtered_draws = 0
-    start = time.time()
-    while (
-        count < 10 and time.time() < start + 1 and
-        filtered_draws < 50 and overruns < 20
-    ):
-        try:
-            data = ConjectureData(
-                max_length=settings.buffer_size,
-                draw_bytes=lambda data, n: uniform(health_check_random, n)
-            )
-            with Settings(settings, verbosity=Verbosity.quiet):
-                test_runner(data, reify_and_execute(
-                    search_strategy,
-                    lambda *args, **kwargs: None,
-                ))
-            count += 1
-        except UnsatisfiedAssumption:
-            filtered_draws += 1
-        except StopTest:
-            if data.status == Status.INVALID:
-                filtered_draws += 1
-            else:
-                assert data.status == Status.OVERRUN
-                overruns += 1
-        except InvalidArgument:
-            raise
-        except Exception:
-            escalate_hypothesis_internal_error()
-            if (
-                HealthCheck.exception_in_generation in
-                settings.suppress_health_check
-            ):
-                raise
-            report(traceback.format_exc())
-            if test_runner is default_new_style_executor:
-                fail_health_check(
-                    settings,
-                    'An exception occurred during data '
-                    'generation in initial health check. '
-                    'This indicates a bug in the strategy. '
-                    'This could either be a Hypothesis bug or '
-                    "an error in a function you've passed to "
-                    'it to construct your data.',
-                    HealthCheck.exception_in_generation,
-                )
-            else:
-                fail_health_check(
-                    settings,
-                    'An exception occurred during data '
-                    'generation in initial health check. '
-                    'This indicates a bug in the strategy. '
-                    'This could either be a Hypothesis bug or '
-                    'an error in a function you\'ve passed to '
-                    'it to construct your data. Additionally, '
-                    'you have a custom executor, which means '
-                    'that this could be your executor failing '
-                    'to handle a function which returns None. ',
-                    HealthCheck.exception_in_generation,
-                )
-    if overruns >= 20 or (
-        not count and overruns > 0
-    ):
-        fail_health_check(settings, (
-            'Examples routinely exceeded the max allowable size. '
-            '(%d examples overran while generating %d valid ones)'
-            '. Generating examples this large will usually lead to'
-            ' bad results. You should try setting average_size or '
-            'max_size parameters on your collections and turning '
-            'max_leaves down on recursive() calls.') % (
-            overruns, count
-        ), HealthCheck.data_too_large)
-    if filtered_draws >= 50 or (
-        not count and filtered_draws > 0
-    ):
-        fail_health_check(settings, (
-            'It looks like your strategy is filtering out a lot '
-            'of data. Health check found %d filtered examples but '
-            'only %d good ones. This will make your tests much '
-            'slower, and also will probably distort the data '
-            'generation quite a lot. You should adapt your '
-            'strategy to filter less. This can also be caused by '
-            'a low max_leaves parameter in recursive() calls') % (
-            filtered_draws, count
-        ), HealthCheck.filter_too_much)
-    runtime = time.time() - start
-    if runtime > 1.0 or count < 10:
-        fail_health_check(settings, (
-            'Data generation is extremely slow: Only produced '
-            '%d valid examples in %.2f seconds (%d invalid ones '
-            'and %d exceeded maximum size). Try decreasing '
-            "size of the data you're generating (with e.g."
-            'average_size or max_leaves parameters).'
-        ) % (count, runtime, filtered_draws, overruns),
-            HealthCheck.too_slow,
-        )
-
-
 def get_random_for_wrapped_test(test, wrapped_test):
     settings = wrapped_test._hypothesis_internal_use_settings
     wrapped_test._hypothesis_internal_use_generated_seed = None
@@ -529,9 +340,6 @@ def new_given_argspec(original_argspec, generator_kwargs):
         args=new_args, kwonlyargs=new_kwonlyargs, annotations=annots)
 
 
-HUNG_TEST_TIME_LIMIT = 5 * 60
-
-
 ROOT = os.path.dirname(__file__)
 
 STDLIB = os.path.dirname(os.__file__)
@@ -580,54 +388,29 @@ FORCE_PURE_TRACER = os.getenv('HYPOTHESIS_FORCE_PURE_TRACER') == 'true'
 
 class StateForActualGivenExecution(object):
 
-    def __init__(self, test_runner, search_strategy, test, settings, random):
+    def __init__(
+        self, test_runner, search_strategy, test, settings, random, had_seed
+    ):
         self.test_runner = test_runner
         self.search_strategy = search_strategy
         self.settings = settings
         self.at_least_one_success = False
         self.last_exception = None
-        self.repr_for_last_exception = None
         self.falsifying_examples = ()
         self.__was_flaky = False
         self.random = random
         self.__warned_deadline = False
         self.__existing_collector = None
         self.__test_runtime = None
-        self.__in_final_replay = False
+        self.__had_seed = had_seed
 
-        if self.settings.deadline is None:
-            self.test = test
-        else:
-            @proxies(test)
-            def timed_test(*args, **kwargs):
-                self.__test_runtime = None
-                start = time.time()
-                result = test(*args, **kwargs)
-                runtime = (time.time() - start) * 1000
-                self.__test_runtime = runtime
-                if self.settings.deadline is not_set:
-                    if (
-                        not self.__warned_deadline and
-                        runtime >= 200
-                    ):
-                        self.__warned_deadline = True
-                        note_deprecation((
-                            'Test took %.2fms to run. In future the default '
-                            'deadline setting will be 200ms, which will '
-                            'make this an error. You can set deadline to '
-                            'an explicit value of e.g. %d to turn tests '
-                            'slower than this into an error, or you can set '
-                            'it to None to disable this check entirely.') % (
-                                runtime, ceil(runtime / 100) * 100,
-                        ))
-                elif runtime >= self.current_deadline:
-                    raise DeadlineExceeded(runtime, self.settings.deadline)
-                return result
-            self.test = timed_test
+        self.test = test
 
         self.coverage_data = CoverageData()
         self.files_to_propagate = set()
 
+        self.used_examples_from_database = False
+
         if settings.use_coverage and not IN_COVERAGE_TESTS:  # pragma: no cover
             if Collector._collectors:
                 self.hijack_collector(Collector._collectors[-1])
@@ -649,13 +432,106 @@ class StateForActualGivenExecution(object):
         else:
             self.collector = None
 
-    @property
-    def current_deadline(self):
-        base = self.settings.deadline
-        if self.__in_final_replay:
-            return base
+    def execute(
+        self, data,
+        print_example=False,
+        is_final=False,
+        expected_failure=None, collect=False,
+    ):
+        text_repr = [None]
+        if self.settings.deadline is None:
+            test = self.test
         else:
-            return base * 1.25
+            @proxies(self.test)
+            def test(*args, **kwargs):
+                self.__test_runtime = None
+                initial_draws = len(data.draw_times)
+                start = benchmark_time()
+                result = self.test(*args, **kwargs)
+                finish = benchmark_time()
+                internal_draw_time = sum(data.draw_times[initial_draws:])
+                runtime = (finish - start - internal_draw_time) * 1000
+                self.__test_runtime = runtime
+                if self.settings.deadline is not_set:
+                    if (
+                        not self.__warned_deadline and
+                        runtime >= 200
+                    ):
+                        self.__warned_deadline = True
+                        note_deprecation((
+                            'Test took %.2fms to run. In future the default '
+                            'deadline setting will be 200ms, which will '
+                            'make this an error. You can set deadline to '
+                            'an explicit value of e.g. %d to turn tests '
+                            'slower than this into an error, or you can set '
+                            'it to None to disable this check entirely.') % (
+                                runtime, ceil(runtime / 100) * 100,
+                        ))
+                else:
+                    current_deadline = self.settings.deadline
+                    if not is_final:
+                        current_deadline *= 1.25
+                    if runtime >= current_deadline:
+                        raise DeadlineExceeded(runtime, self.settings.deadline)
+                return result
+
+        def run(data):
+            with self.settings:
+                with BuildContext(data, is_final=is_final):
+                    import random as rnd_module
+                    rnd_module.seed(0)
+                    args, kwargs = data.draw(self.search_strategy)
+                    if expected_failure is not None:
+                        text_repr[0] = arg_string(test, args, kwargs)
+
+                    if print_example:
+                        report(
+                            lambda: 'Falsifying example: %s(%s)' % (
+                                test.__name__, arg_string(test, args, kwargs)))
+                    elif current_verbosity() >= Verbosity.verbose:
+                        report(
+                            lambda: 'Trying example: %s(%s)' % (
+                                test.__name__, arg_string(test, args, kwargs)))
+
+                    if self.collector is None or not collect:
+                        return test(*args, **kwargs)
+                    else:  # pragma: no cover
+                        try:
+                            self.collector.start()
+                            return test(*args, **kwargs)
+                        finally:
+                            self.collector.stop()
+
+        result = self.test_runner(data, run)
+        if expected_failure is not None:
+            exception, traceback = expected_failure
+            if (
+                isinstance(
+                    exception,
+                    DeadlineExceeded
+                ) and self.__test_runtime is not None
+            ):
+                report((
+                    'Unreliable test timings! On an initial run, this '
+                    'test took %.2fms, which exceeded the deadline of '
+                    '%.2fms, but on a subsequent run it took %.2f ms, '
+                    'which did not. If you expect this sort of '
+                    'variability in your test timings, consider turning '
+                    'deadlines off for this test by setting deadline=None.'
+                ) % (
+                    exception.runtime,
+                    self.settings.deadline, self.__test_runtime
+                ))
+            else:
+                report(
+                    'Failed to reproduce exception. Expected: \n' +
+                    traceback,
+                )
+            self.__flaky((
+                'Hypothesis %s(%s) produces unreliable results: Falsified'
+                ' on the first call but did not on a subsequent one'
+            ) % (test.__name__, text_repr[0],))
+        return result
 
     def should_trace(self, original_filename, frame):  # pragma: no cover
         disp = FileDisposition()
@@ -701,20 +577,9 @@ class StateForActualGivenExecution(object):
         collector.save_data = save_data
 
     def evaluate_test_data(self, data):
-        if (
-            time.time() - self.start_time >= HUNG_TEST_TIME_LIMIT
-        ):
-            fail_health_check(self.settings, (
-                'Your test has been running for at least five minutes. This '
-                'is probably not what you intended, so by default Hypothesis '
-                'turns it into an error.'
-            ), HealthCheck.hung_test)
-
         try:
             if self.collector is None:
-                result = self.test_runner(data, reify_and_execute(
-                    self.search_strategy, self.test,
-                ))
+                result = self.execute(data)
             else:  # pragma: no cover
                 # This should always be a no-op, but the coverage tracer has
                 # a bad habit of resurrecting itself.
@@ -722,10 +587,7 @@ class StateForActualGivenExecution(object):
                 sys.settrace(None)
                 try:
                     self.collector.data = {}
-                    result = self.test_runner(data, reify_and_execute(
-                        self.search_strategy, self.test,
-                        collector=self.collector
-                    ))
+                    result = self.execute(data, collect=True)
                 finally:
                     sys.settrace(original)
                     covdata = CoverageData()
@@ -768,7 +630,10 @@ class StateForActualGivenExecution(object):
     def run(self):
         # Tell pytest to omit the body of this function from tracebacks
         __tracebackhide__ = True
-        database_key = str_to_bytes(fully_qualified_name(self.test))
+        if global_force_seed is None:
+            database_key = str_to_bytes(fully_qualified_name(self.test))
+        else:
+            database_key = None
         self.start_time = time.time()
         global in_given
         runner = ConjectureRunner(
@@ -790,6 +655,25 @@ class StateForActualGivenExecution(object):
                 sys.settrace(original_trace)
         note_engine_for_statistics(runner)
         run_time = time.time() - self.start_time
+
+        self.used_examples_from_database = runner.used_examples_from_database
+
+        if runner.used_examples_from_database:
+            if self.settings.derandomize:
+                note_deprecation(
+                    'In future derandomize will imply database=None, but your '
+                    'test is currently using examples from the database. To '
+                    'get the future behaviour, update your settings to '
+                    'include database=None.'
+                )
+            if self.__had_seed:
+                note_deprecation(
+                    'In future use of @seed will imply database=None in your '
+                    'settings, but your test is currently using examples from '
+                    'the database. To get the future behaviour, update your '
+                    'settings for this test to include database=None.'
+                )
+
         timed_out = runner.exit_reason == ExitReason.timeout
         if runner.last_data is None:
             return
@@ -843,81 +727,28 @@ class StateForActualGivenExecution(object):
 
         flaky = 0
 
-        self.__in_final_replay = True
-
         for falsifying_example in self.falsifying_examples:
             self.__was_flaky = False
-            raised_exception = False
+            assert falsifying_example.__expected_exception is not None
             try:
-                with self.settings:
-                    self.test_runner(
-                        ConjectureData.for_buffer(falsifying_example.buffer),
-                        reify_and_execute(
-                            self.search_strategy, self.test,
-                            print_example=True, is_final=True
-                        ))
+                self.execute(
+                    ConjectureData.for_buffer(falsifying_example.buffer),
+                    print_example=True, is_final=True,
+                    expected_failure=(
+                        falsifying_example.__expected_exception,
+                        falsifying_example.__expected_traceback,
+                    )
+                )
             except (UnsatisfiedAssumption, StopTest):
                 report(traceback.format_exc())
                 self.__flaky(
                     'Unreliable assumption: An example which satisfied '
                     'assumptions on the first run now fails it.'
                 )
-            except:
+            except BaseException:
                 if len(self.falsifying_examples) <= 1:
                     raise
-                raised_exception = True
                 report(traceback.format_exc())
-
-            if not raised_exception:
-                if (
-                    isinstance(
-                        falsifying_example.__expected_exception,
-                        DeadlineExceeded
-                    ) and self.__test_runtime is not None
-                ):
-                    report((
-                        'Unreliable test timings! On an initial run, this '
-                        'test took %.2fms, which exceeded the deadline of '
-                        '%.2fms, but on a subsequent run it took %.2f ms, '
-                        'which did not. If you expect this sort of '
-                        'variability in your test timings, consider turning '
-                        'deadlines off for this test by setting deadline=None.'
-                    ) % (
-                        falsifying_example.__expected_exception.runtime,
-                        self.settings.deadline, self.__test_runtime
-                    ))
-                else:
-                    report(
-                        'Failed to reproduce exception. Expected: \n' +
-                        falsifying_example.__expected_traceback,
-                    )
-
-                filter_message = (
-                    'Unreliable test data: Failed to reproduce a failure '
-                    'and then when it came to recreating the example in '
-                    'order to print the test data with a flaky result '
-                    'the example was filtered out (by e.g. a '
-                    'call to filter in your strategy) when we didn\'t '
-                    'expect it to be.'
-                )
-
-                try:
-                    self.test_runner(
-                        ConjectureData.for_buffer(falsifying_example.buffer),
-                        reify_and_execute(
-                            self.search_strategy,
-                            test_is_flaky(
-                                self.test, self.repr_for_last_exception),
-                            print_example=True, is_final=True
-                        ))
-                except (UnsatisfiedAssumption, StopTest):
-                    self.__flaky(filter_message)
-                except Flaky as e:
-                    if len(self.falsifying_examples) > 1:
-                        self.__flaky(e.args[0])
-                    else:
-                        raise
-
             if self.__was_flaky:
                 flaky += 1
 
@@ -977,6 +808,17 @@ def given(*given_arguments, **given_kwargs):
             # Tell pytest to omit the body of this function from tracebacks
             __tracebackhide__ = True
 
+            if getattr(test, 'is_hypothesis_test', False):
+                note_deprecation((
+                    'You have applied @given to a test more than once. In '
+                    'future this will be an error. Applying @given twice '
+                    'wraps the test twice, which can be extremely slow. A '
+                    'similar effect can be gained by combining the arguments '
+                    'to the two calls to given. For example, instead of '
+                    '@given(booleans()) @given(integers()), you could write '
+                    '@given(booleans(), integers())'
+                ))
+
             settings = wrapped_test._hypothesis_internal_use_settings
 
             random = get_random_for_wrapped_test(test, wrapped_test)
@@ -1011,16 +853,18 @@ def given(*given_arguments, **given_kwargs):
                 return
 
             try:
-                perform_health_checks(
-                    random, settings, test_runner, search_strategy)
-
                 state = StateForActualGivenExecution(
-                    test_runner, search_strategy, test, settings, random)
+                    test_runner, search_strategy, test, settings, random,
+                    had_seed=wrapped_test._hypothesis_internal_use_seed
+                )
                 state.run()
-            except:
+            except BaseException:
                 generated_seed = \
                     wrapped_test._hypothesis_internal_use_generated_seed
-                if generated_seed is not None:
+                if (
+                    generated_seed is not None and
+                    not state.used_examples_from_database
+                ):
                     if running_under_pytest:
                         report((
                             'You can add @seed(%(seed)d) to this test or run '
@@ -1055,6 +899,7 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
         min_satisfying_examples=0,
         max_shrinks=2000,
     )
+    settings = Settings(settings, perform_health_check=False)
 
     if database_key is None and settings.database is not None:
         database_key = function_digest(condition)
