@@ -23,6 +23,7 @@ import os
 import re
 import sys
 import math
+import time
 import codecs
 import platform
 import importlib
@@ -114,7 +115,8 @@ def quiet_raise(exc):
     struct_pack = struct.pack
     struct_unpack = struct.unpack
 
-    from time import monotonic as benchmark_time
+    def benchmark_time():
+        return time.monotonic()
 else:
     import struct
 
@@ -240,7 +242,8 @@ else:
     def quiet_raise(exc):
         raise exc
 
-    from time import time as benchmark_time
+    def benchmark_time():
+        return time.time()
 
 
 # coverage mixes unicode and str filepaths on Python 2, which causes us
@@ -294,22 +297,6 @@ if PY2:
 else:
     from inspect import getfullargspec, FullArgSpec
 
-    if sys.version_info[:2] == (3, 5):
-        # silence deprecation warnings on Python 3.5
-        # (un-deprecated in 3.6 to allow single-source 2/3 code like this)
-        def silence_warnings(func):
-            import warnings
-            import functools
-
-            @functools.wraps(func)
-            def inner(*args, **kwargs):
-                with warnings.catch_warnings():
-                    warnings.simplefilter('ignore', DeprecationWarning)
-                    return func(*args, **kwargs)
-            return inner
-
-        getfullargspec = silence_warnings(getfullargspec)
-
 
 if sys.version_info[:2] < (3, 6):
     def get_type_hints(thing):
@@ -507,8 +494,6 @@ else:
 # an existing file where you're not allowed to. This is rather less consistent
 # between versions than might be hoped.
 if PY3:
-    # Note: This only works on >= 3.3, but we only support >= 3.3 so that's
-    # fine
     FileExistsError = FileExistsError
 
 elif WINDOWS:
